@@ -4,13 +4,13 @@ use warnings;
 use strict;
 use Carp;
 
-use version; our $VERSION = qv('0.1.0');
+use version; our $VERSION = qv('0.3.0');
 
 use Algorithm::Combinatorics qw(variations_with_repetition);
 
 our @ISA = qw(Exporter);
 
-our @EXPORT_OK = qw( check_combination partitions );
+our @EXPORT_OK = qw( check_combination partitions entropy );
 
 use lib qw( ../../lib ../lib ../../../lib );
 
@@ -124,6 +124,67 @@ sub check_combination {
   my $combination = shift;
   my $string = shift;
 
+  my ( %hash_combination, %hash_string );
+  my $blacks = 0;
+  my ($c, $s);
+  while ( $c = chop( $combination ) ) {
+    $s = chop( $string );
+    if ( $c eq $s ) {
+      $blacks++;
+    } else {
+      $hash_combination{ $c }++;
+      $hash_string{ $s }++;
+    }
+  }
+  my $whites = 0;
+  for my $k ( keys %hash_combination ) {
+    next if ! defined $hash_string{$k};
+    $whites += ($hash_combination{$k} > $hash_string{$k})
+      ?$hash_string{$k}
+	:$hash_combination{$k};
+  }
+  return { blacks => $blacks,
+	   whites => $whites };
+}
+
+sub distance_taxicab {
+  my $self = shift;
+  my $combination = shift || croak "Can't compute distance to nothing";
+  my $rules =  $self->number_of_rules();
+  my $matches = $self->matches( $combination );
+
+  my $distance = 0;
+  my @rules = @{$self->{'_rules'}};
+  for ( my $r = 0; $r <= $#rules; $r++) {
+    $distance -= abs( $rules[$r]->{'blacks'} - $matches->{'result'}->[$r]->{'blacks'} ) +
+      abs( $rules[$r]->{'whites'} - $matches->{'result'}->[$r]->{'whites'} );
+  }
+
+  return [$distance, $matches->{'matches'}];
+}
+
+sub distance_chebyshev {
+  my $self = shift;
+  my $combination = shift || croak "Can't compute distance to nothing";
+  my $rules =  $self->number_of_rules();
+  my $matches = $self->matches( $combination );
+
+  my $distance = 0;
+  my @rules = @{$self->{'_rules'}};
+  for ( my $r = 0; $r <= $#rules; $r++) {
+    my $diff_black = abs( $rules[$r]->{'blacks'} - $matches->{'result'}->[$r]->{'blacks'});
+    my $diff_white = abs( $rules[$r]->{'whites'} - $matches->{'result'}->[$r]->{'whites'} );
+    my $this_distance = ($diff_black > $diff_white)?$diff_black:$diff_white;
+    $distance -= $this_distance ;
+  }
+
+  return [$distance, $matches->{'matches'}];
+}
+
+sub check_combination_old {
+  my $combination = shift;
+  my $string = shift;
+
   my @combination_arr = split(//, $combination );
   my @string_arr = split(//, $string);
   my $blacks = 0;
@@ -192,6 +253,42 @@ sub all_combinations {
   
 }
 
+sub all_responses {
+    my $self = shift;
+    my $length = $self->{'_length'};
+    my @responses_array = variations_with_repetition( ['B', 'W', '-'], 
+						      $length );
+    my %responses;
+    for my $r ( @responses_array ) {
+      my %partial = ( W => 0,
+		      B => 0 );
+      for my $c (@$r) {
+	$partial{$c}++;
+      }
+      
+      $responses{$partial{'B'}."B-".$partial{'W'}."W"} = 1;
+    }
+    # Delete impossible
+    my $impossible = ($length-1)."B-1W";
+    delete $responses{$impossible};
+    my @possible_responses = sort keys %responses;
+    return @possible_responses;
+
+}
+
+sub entropy {
+  my $combination = shift;
+  my %freqs;
+  map( $freqs{$_}++, split( //, $combination));
+  my $entropy;
+  for my $k (keys %freqs ) {
+    my $probability = $freqs{$k}/length($combination);
+    $entropy -= $probability * log ($probability);
+  }
+  return $entropy;
+}
+
+
 "4 blacks, 0 white"; # Magic true value required at end of module
 
 __END__
@@ -245,6 +342,18 @@ Checks a combination against the secret code, returning a hashref with
 the number of blacks (correct in position) and whites (correct in
 color, not position)
 
+=head2 distance( $object )
+
+Computes distance to a consistent combination, computed as the number
+of blacks and whites that need change to become a consistent
+combination. 
+
+
+=head2 check_combination_old ( $secret_code,
+$combination )
+
+Old way of checking combinations, eliminated after profiling
+
 =head2 check_rule ($rule, $combination) 
 
 Same as C<check_combination>, except that a rule contains a
@@ -320,6 +429,24 @@ Returns all possible combinations of the current alphabet and length
 in an array. Be careful with that, it could very easily fill up your
 memory, depending on length and alphabet size.
 
+=head2 entropy( $string )
+
+Computes the string entropy
+
+=head2 distance_taxicab( $string )
+
+Computes the sums of taxicab distances to all combinations in the
+game, and returns it as [$distance, $matches]
+
+=head2 distance_chebyshev( $string )
+
+Computes the Chebyshev distance, that is, the max of distances in all
+dimensions. Returns as a arrayref with [$distance, matches]
+
+=head2 all_responses()
+
+Returns all possible responses (combination of black and white pegs)
+for the combination length
 
 =head1 CONFIGURATION AND ENVIRONMENT
 
@@ -328,7 +455,9 @@ Algorithm::MasterMind requires no configuration files or environment variables.
 
 =head1 DEPENDENCIES
 
-L<Algorithm::Evolutionary>, but only for one of the strategies.
+L<Algorithm::Evolutionary>, but only for one of the
+strategies. L<Algorithm::Combinatorics>, used to generate combinations
+and for exhaustive search strategies. 
 
 
 =head1 INCOMPATIBILITIES
@@ -359,6 +488,20 @@ The development of this projects is hosted at sourceforge,
 https://sourceforge.net/projects/opeal/develop, check it out for the
     latest bleeding edge release. 
 
+If you use any of these modules for your own research, we would very
+grateful if you would reference the papers that describe this, such as
+this one:
+
+ @article{merelo2010finding,
+  title={{Finding Better Solutions to the Mastermind Puzzle Using Evolutionary Algorithms}},
+  author={Merelo-Guerv{\'o}s, J. and Runarsson, T.},
+  journal={Applications of Evolutionary Computation},
+  pages={121--130},
+  year={2010},
+  publisher={Springer}
+ }
+
+
 =head1 AUTHOR
 
 JJ Merelo  C<< <jj@merelo.net> >>
